import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;

/**
 * Die Klasse Feld repraesentiert die Felder in einem Schachbrett-
 * artigen Gesamtfeld zur Suche mit A*
 * 
 * @author (claus albowski) 
 * @version (Juni 15)
 */
public class Feld
{
    private int zeile,spalte; //seine Koordinaten 1 basierend
    private String farbe;
    private boolean gesperrt;
    private ArrayList<Shape> teilfiguren;
    private static int basisLaenge=30;
    private static int kantenlaenge;
    private static ArrayList<Feld> gesamtFeld = new ArrayList();
    private static ArrayList<Feld> besuchtListe = new ArrayList();

    // ********** Konstruktor **********
    
    /**
     * Konstruktor für Objekte der Klasse Feld
     */
    public Feld(int zeile, int spalte)
    {
        this.zeile = zeile;
        this.spalte = spalte;
        farbe = "schwarz";
        gesperrt = false;
        teilfiguren = new ArrayList();
        gesamtFeld.add(this);
        Rectangle2D rechteck = new Rectangle2D.Double(
                gibSpaltenPosition(), gibZeilenPosition(),
                basisLaenge, basisLaenge);
        teilfiguren.add(rechteck);
    }
    
    // ********** Klassenmethoden **********

    public static void erzeugeFelder(int kantenlaenge)
    {
        Feld.kantenlaenge = kantenlaenge;
        for (int zeile=1; zeile<kantenlaenge+1; zeile++)
        for (int spalte=1; spalte<kantenlaenge+1; spalte++)
        {
            new Feld(zeile,spalte);
            gibFeld(zeile, spalte).zeichne();
        }
    }
    
    /**
     * gibt zurueck, ob zeile und spalte zulaessig sind
     */
    public static boolean zulaessig(int zeile, int spalte)
    {
        if (zeile<=0) return false;
        if (zeile>kantenlaenge) return false;
        if (spalte<=0) return false;
        if (spalte>kantenlaenge) return false;
        return true;
    }
    
    /**
     * ermoeglicht den Zugriff auf die einzelnen Felder
     * zeile: 1..kantenlaenge / spalte:1..kantenlaenge
     */
    public static Feld gibFeld(int zeile, int spalte)
    {
        int index = (zeile-1)*kantenlaenge+spalte-1;
        if (zulaessig(zeile,spalte)&&(gesamtFeld.size()>index))
        return gesamtFeld.get(index);
        else return null;
    }
    
    /**
     * kennzeichnet das Feld mit den uebergebenen Koordinaten
     * als gesperrt;
     * benutzt dazu die Instanzmethode
     */
    public static void sperre(int zeile, int spalte)
    {
        Feld.gibFeld(zeile, spalte).sperre();
    }
    
    /**
     * gibt zu einem Feld die zulaessigen Nachbarfelder zurueck
     * laesst auch die gesperrten aus
     */
    public static ArrayList<Feld> gibNachbarn(Feld feld)
    {
        int z = feld.gibZeile();
        int s = feld.gibSpalte();
        int[][] indexPaare = { {z-1,s}, {z+1,s}, {z,s-1}, {z,s+1}};
        ArrayList<Feld> nachbarn = new ArrayList();
        for (int i=0; i<4; i++)
        if (Feld.zulaessig(indexPaare[i][0], indexPaare[i][1]))
        nachbarn.add(Feld.gibFeld(indexPaare[i][0], indexPaare[i][1]));
        return nachbarn;
    }
    
//     /**
//      * kennzeichnet das uebergebene Feld als besucht
//      */
//     public static void besucht(Feld feld)
//     {
//         besuchtListe.add(feld);
//     }
        
    // ********** Instanzmethoden **********
    
    public int gibZeile() { return zeile; }
    public int gibSpalte() { return spalte; }
    public boolean istGesperrt() { return gesperrt; }
    public boolean equals(Feld feld)
    {
        if (gibZeile()!=feld.gibZeile()) return false;
        if (gibSpalte()!=feld.gibSpalte()) return false;
        return true;
    }
    
    /**
     * kennzeichnet das Feld als gesperrt
     */
    public void sperre()
    {
        loesche();
        gesperrt = true;
        Shape kreis = new Ellipse2D.Double(
              gibSpaltenPosition()+basisLaenge*0.15,
              gibZeilenPosition()+basisLaenge*0.15,
              basisLaenge*0.7,
              basisLaenge*0.7);
        teilfiguren.add(kreis);
        zeichne();
    }
    
    /**
     * kennzeichnet das uebergebene Feld als besucht
     */
    public void besucht()
    {
        besuchtListe.add(this);
    }
    
    /**
     * zeigt an, ob das Feld besucht ist
     */
    public boolean istBesucht()
    {
        return besuchtListe.contains(this);
    }
    
    /**
     * fuegt eine Beschriftung ein
     */
    public void beschrifte(String text)
    {
        Leinwand.gibLeinwand().beschrifteMit(text,
                    gibSpaltenPosition()+basisLaenge/3,
                    gibZeilenPosition()+2*basisLaenge/3);
    }

    /**
     * rechnet die zeile-Koordinate in die Position der Darstellung um
     */
    private int gibZeilenPosition()
    {
        return (1 + zeile) * basisLaenge;
    }
    
    /**
     * rechnet die spalte-Koordinate in die Position der Darstellung um
     */
    private int gibSpaltenPosition()
    {
        return (1 + spalte) * basisLaenge;
    }
    
    /**
     * berechnet die Manhattan-Distanz der Koordinaten
     * der beiden Felder
     */
    public int manhattanDistanz(Feld andere)
    {
        int dx = Math.abs(zeile-andere.gibZeile());
        int dy = Math.abs(spalte-andere.gibSpalte());
        return dx+dy;
    }

    // ********** Grafikabschnitt **********
    
    /**
     * Berechnet den zu zeichnenden Graphen anhand der gegebenen Daten.
     */
    private Shape gibAktuelleFigur()
    {
        GeneralPath graph = new GeneralPath();
        for (Shape figur : teilfiguren)
        graph.append(figur, false);
        return graph;
    }
  
    /**
     * stellt das Feld dar.
     */
    public void zeichne()
    {
        Shape figur = gibAktuelleFigur();
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.zeichne (
          this,           // leinwand kennt das Objekt
          farbe,          // definiert seine Zeichenfarbe
          figur);         // definiert seinen grafischen Aspekt
    }

    /**
     * Loesche dies Feld vom Bildschirm.
     */
    public void loesche()
    {
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.entferne(this);
    }
}
